/* sine.c
 *
 * Outputs a sine wave at 440Hz out the left channel of the audio board
 */

#include <xparameters.h>
#include <xintc_l.h>


/* the low level driver for the audio */
#include "audio_l.h"

#define HIGH_VALUE (20)
#define LOW_VALUE (-20)

/* This makes it so that it swings most of the way through a 24-bit number
since 20 is represented by 6 bits */
#define SHIFT_AMOUNT (24 - 6)


int curr_value;
int climbing;

void audio_interrupt_handler(void *InstancePtr)
{
	/* 
	* TODO: calculate the next sample and give it to the audio controller. 
	*
	* Note that this interrupt will happen every 23 microseconds, or
	* at 43.4kHz (the sample rate of the codec)
	*/

	/* this currently makes a triangle wave */
	if(curr_value == HIGH_VALUE) climbing = 0;
	if(curr_value == LOW_VALUE) climbing = 1;
	if(climbing) 
	{
		curr_value++;
	}
	else
	{
		curr_value--;
	}
	
	XAudio_mWriteOutput(XPAR_AUDIO_BASEADDR, LEFT, 0, curr_value << SHIFT_AMOUNT);
}


int main() 
{

	/* TODO: initialization code should go here */

	curr_value = 0;
	climbing = 1;


    /* register for the interrupts */
    XIntc_InterruptVectorTable[0].Handler = audio_interrupt_handler;
    XIntc_InterruptVectorTable[0].CallBackRef = NULL;
    XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_AUDIO_INTERRUPT_MASK);
    XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR);
        
    /* globally enable the interrupts on the microblaze */
    microblaze_enable_interrupts();

	/* enable the audio codec and make it interrupt me every time it wants
	new data */
    XAudio_mSetControlReg(XPAR_AUDIO_BASEADDR, AUDIO_CR_INT_ENABLE_MASK | AUDIO_CR_ENABLE_MASK);
    
    for(;;)
    {
       /* do nothing, just let the interrupts handle the rest of the work */
    }

    return 0;  /* never reached */
}
  
